/*************************************************************************
gsf_mask_calipso  GSF Conversion program for CALIPSO
product.  This program extracts the Total_Attenuated_Backscatter_323 
(currently only this field) into a gsf file

Copyright (C) 2003  Todd Berendes

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

See the LICENSE.txt file for the full GPL License

Todd Berendes 
University of Alabama in Huntsville    
320 Sparkman Drive              
Huntsville, AL  35805            
todd@nsstc.uah.edu             
***************************************************************************/
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include "gsf.h"
#include "hdf.h"
/*must be odd*/
#define LATLON_NEIGHBORHOOD 3
#define FILENAME_SIZE 240
#define	Pi	3.141592654

typedef struct {
   int Argc;
      char **Argv;
      char *InputFileName;
      char *OutputFileName;
      int Errors;
      int overwritePromptFlag;
   } CommandLineStruct;
CommandLineStruct CommandLineInfo;

typedef struct {
    int Year, JulianDay, SatNumber, NumPix;
    int NumNavPts, StartNavPoint, NavPointInterval;
    float SatelliteAltitude, SpatialResolution;
    float Scale, Offset;
}CalipsoMetadataType;

float timePerScan = 1.477145;
float timePerPixel;


float deg2rad = M_PI/180.0;

char *getOneTextObjectValue(char *attrName, char *buffer);
int mmddyy_to_jday(int month, int day, int year);
void	InterpolateLatLong(float *LatBuffer, float *LonBuffer, float *Lat, float *Lon
     , int  NumNavPoints, int NumInterpolatedSamples, int  StartNavPoint, int  NavPointInterval);
void ProcessCommandLine(CommandLineStruct *poCommandLineInfo);
void PrintCommandLineOptions(char *argv[]);
float32 **calloc2dFloat32Array(int dim1, int dim2); 
void dealloc2dFloat32Array(float32 **Buffer, int dim1);
void setupLUT(GSFType *GSFdata);
void extractProfileTime(double profile_time, int *year, int *month, int *day, int *hour, int *minute, double *sec);
double secToHrs(double secondsInDay);

CalipsoMetadataType CalipsoMetadata;
/*float64 *startTime;*/
GSFType GSFdata;
    
main(int argc, char *argv[]) 
{  
int linecnt;
int ind1, ind2, ind3;
FILE *ftemp;
int32	Max_1km_frames;

int32   sd_id, v_id, sds_id, sds_index, file_id, status, n_datasets, n_file_attrs;
int32   index, rank, num_type, attributes, dim_index;
int32   dim_sizes[MAX_VAR_DIMS];
char    sds_name[64];
int32   records_read;
int32   record_pos;
int32   interlace_mode;
int attrIndex;
char attrName[MAX_NC_NAME];
int32 dataType, count;
char *valueBuffer;
char chTest[240];

int year, day, month;
char tempStr[240];
char *tempPtr;
int finalLinesPerScan;
double ftime;
int BadLatCount = 0;
char OutputFileName[240];
char filenameDir[240];
char rangeBeginningDate[240];
char rangeBeginningTime[240];
char rangeEndingTime[240];
float beginTime, endTime, timeInterval;
int hour, minute;
double second;
int32 start[3], edges[3];
/*   
float calib_line[MAS_MAX_CHANNELS][MAS_MAX_PIXELS];
*/
GSFType GSFdataIn;
GSFType GSFdataOut;
float32 *CalipsoData;
float *outputLine;
int numDays;
int modChanInd;
float64 addOffset, scaleFactor;
int32 data_type;
int32 dcount;

float64 *profile_time;
float32 altitude;
float *Lat, *Lon;
float32 *LatBuf, *LonBuf;
int nscans;
double *startTime;

int32 vdata_ref, vdata_id;

char field_name_list[5000];
char vdata_name[VSNAMELENMAX];
int32 n_records;
int32 vdata_size;
float32 *lidarHeights;
int32 field_index;
uint8 fieldBuf[100000];
int32 fieldSize;
VOIDP bufPtrs[1];
TableType *heightTable;

    CommandLineInfo.Argc = argc;
    CommandLineInfo.Argv = argv;
    /* process command line options*/
    ProcessCommandLine(&CommandLineInfo);

    if (CommandLineInfo.Errors) {
	PrintCommandLineOptions(argv);
	exit(-1);
    }

    if ((ftemp=fopen(CommandLineInfo.InputFileName, "r"))==NULL) {
	printf("cannot open file %s\n", CommandLineInfo.InputFileName);
	exit(-1);
    }
    else {
	fclose(ftemp);
    }
    if (Hishdf(CommandLineInfo.InputFileName) == 0) {
      printf("Error: %s is not a valid HDF file \n", CommandLineInfo.InputFileName);
      exit(-1);
    } 
    if ((tempPtr=strstr(CommandLineInfo.OutputFileName,".gsf")) == NULL) {
	strcat(CommandLineInfo.OutputFileName,".gsf");
    }
    printf("output filename %s\n",CommandLineInfo.OutputFileName);

    strcpy(OutputFileName,CommandLineInfo.OutputFileName);
    
    /*
     * Open connections to original (input) HDF file
     */
	 file_id = Hopen (CommandLineInfo.InputFileName, DFACC_READ, 0);
	 if ( file_id == -1 ) {
            printf("Error opening input file : %s\n", CommandLineInfo.InputFileName);
            exit(-1);
	 }


    /*
     * Open HDF SDS and Vdata Interfaces
     */
	sd_id = SDstart(CommandLineInfo.InputFileName, DFACC_READ);
	if (sd_id == -1) {
            printf ("SDstart failed.\n");
            HEprint (stdout, 0);
            exit (-1);
	}        
        
    /*
     * Obtain number of SDSs and Global Attributes in input file
     */

	status = SDfileinfo(sd_id, &n_datasets, &n_file_attrs);
	if (status != 0 ) {
            printf("SDfileinfo failed. \n");
            exit (-1);
	}
    
    /*
     * List out SDSs contained in input HDF file
     */
	printf("\n Number of SDS arrays in file: %d\n\n",n_datasets);

	for (index = 0; index < n_datasets; index++) {
             sds_id = SDselect(sd_id, index);

             status = SDgetinfo(sds_id, sds_name, &rank, dim_sizes, &num_type, &attributes);
             if (status != 0) {        
        	printf ("SDgetinfo failed.\n");
        	HEprint (stdout, 0);
        	exit (-1);
             }

             printf("   %2d) %-25s   dimensions = ", index+1, sds_name);
             for(dim_index = 0; dim_index < rank; dim_index++)
        	 printf("%7d", dim_sizes[dim_index]);
             printf(" \n");

             status = SDendaccess(sds_id);
	 }

         CalipsoMetadata.SpatialResolution = 1000;
	 printf("spatial resolution %f\n",CalipsoMetadata.SpatialResolution);

	 start[0] = 0;
	 start[1] = 0;
	 start[2] = 0;
         
 /* need to find number of pixels (nscans)*/
                 
 	 /* info on lidar */
	 sds_index = SDnametoindex(sd_id, "Total_Attenuated_Backscatter_532");
	 if (sds_index == FAIL) {
	     printf("Total_Attenuated_Backscatter_532\n");
	     exit(-1);
	 }
	 sds_id = SDselect(sd_id, sds_index);
	 status = SDgetinfo(sds_id, sds_name, &rank, dim_sizes, &num_type, &attributes);
	 if (status != 0) {        
            printf ("SDgetinfo failed.\n");
            HEprint (stdout, 0);
            exit (-1);
	 }
         nscans = dim_sizes[0];
         CalipsoMetadata.NumPix = dim_sizes[1];
         CalipsoMetadata.NumNavPts = CalipsoMetadata.NumPix;
 	 status = SDendaccess(sds_id);
         printf("numPix %d numLines %d\n",CalipsoMetadata.NumPix, nscans);
    	 Lat = (float *) calloc(CalipsoMetadata.NumPix, sizeof(float));
    	 Lon = (float *) calloc(CalipsoMetadata.NumPix, sizeof(float));
 	 status = SDendaccess(sds_id);
         
         lidarHeights = (float32 *) calloc(CalipsoMetadata.NumPix, sizeof(float32));
         bufPtrs[0]=lidarHeights;
        
	status = Vstart (file_id);
	if (status == -1 ) {
            printf ("Vstart failed.\n");
            HEprint (stdout, 0);
            exit (-1);
	}

        vdata_ref = -1;
        printf("Vdata fields\n");
        while (1) {
            status = VSgetid(file_id,vdata_ref);
            if (status<0) break;
            vdata_ref = status;
            vdata_id = VSattach(file_id, vdata_ref, "r");
            VSinquire(vdata_id, &n_records, &interlace_mode, field_name_list, &vdata_size, vdata_name);
            status = VSdetach(vdata_id);
            printf("Name %s n_records %d size %d", vdata_name, n_records, vdata_size);
            printf("  field names %s\n",field_name_list);
        }
        
        vdata_ref = VSfind(file_id, "metadata");
        vdata_id = VSattach(file_id, vdata_ref, "r");
        VSinquire(vdata_id, &n_records, &interlace_mode, field_name_list, &vdata_size, vdata_name);

/*                
        status = VSfindex(vdata_id, "Lidar_Data_Altitudes", &field_index);
	 if (status != 0) {        
            printf ("Lidar_Data_Altitudes VSfindex failed.\n");
            HEprint (stdout, 0);
            exit (-1);
	 }
         
*/
        
        status = VSsetfields(vdata_id, "Lidar_Data_Altitudes");
	 if (status != 0) {        
            printf ("VSSetfields failed.\n");
            HEprint (stdout, 0);
            exit (-1);
	 }
        fieldSize = VSsizeof(vdata_id,"Lidar_Data_Altitudes");
        printf("lidar size %d\n",fieldSize);
        status = VSread(vdata_id, fieldBuf, 1, interlace_mode);
	 if (status < 0) {        
            printf ("VSread failed.\n");
            HEprint (stdout, 0);
            exit (-1);
	 }
         printf("VSread %d records\n", status);
        status = VSfpack(vdata_id, _HDF_VSUNPACK , "Lidar_Data_Altitudes", (VOIDP) fieldBuf, fieldSize, 1, "Lidar_Data_Altitudes", bufPtrs);
	 if (status != 0) {        
            printf ("VSfpack failed.\n");
            HEprint (stdout, 0);
            exit (-1);
	 }

        status = VSdetach(vdata_id);
	Vend(sd_id);
/*        
        for(ind1=0;ind1<CalipsoMetadata.NumPix;ind1++){
            printf("height %d %f\n", ind1, lidarHeights[ind1]);
        }
*/
        
                
         
 	 printf("number of scans %d\n",dim_sizes[0]);
   	 /* get date */
	 sds_index = SDnametoindex(sd_id, "Profile_UTC_Time");
	 if (sds_index == FAIL) {
	     printf("cannot find Profile_UTC_Time in file\n");
	     exit(-1);
	 }
	 sds_id = SDselect(sd_id, sds_index);
	 status = SDgetinfo(sds_id, sds_name, &rank, dim_sizes, &num_type, &attributes);
	 if (status != 0) {        
            printf ("SDgetinfo failed.\n");
            HEprint (stdout, 0);
            exit (-1);
	 }
        /* printf("rank %d num_type %d\n",rank, num_type);*/
	 profile_time = (float64 *)calloc(dim_sizes[0], sizeof(float64));
	 if (!profile_time) {
	     printf("cannot calloc for profile_time\n");
	     exit(-1);
	 }
	 edges[0] = dim_sizes[0];
	 edges[1] = 1;
	 edges[2] = 0;
	 status = SDreaddata(sds_id, start, NULL, edges, (VOIDP)profile_time);
	 if (status != 0) {    
             printf("read profile_time failed\n");
             exit(-1);
         }    
    	 startTime = (double *) calloc(dim_sizes[0], sizeof(double));
         for (ind1=0;ind1<dim_sizes[0];ind1++) {
             extractProfileTime(profile_time[ind1], &year, &month, &day, &hour, &minute, &second); 
             startTime[ind1] = (double)hour + (double)minute/60.0 + second/3600.0;
         }
	 
 	 status = SDendaccess(sds_id);
        
         extractProfileTime(profile_time[0], &year, &month, &day, &hour, &minute, &second);         
	 CalipsoMetadata.JulianDay = mmddyy_to_jday(month, day, year);	          
	 printf("beginningdate %f:  year %d month %d day %d,  jday %d time: %02d:%02d:%f\n",profile_time[0], year, month, day, CalipsoMetadata.JulianDay, hour, minute, second);

	 /* get spacecraft altitude */
	 sds_index = SDnametoindex(sd_id, "Spacecraft_Altitude");
	 if (sds_index == FAIL) {
	     printf("cannot find Spacecraft_Altitude in file\n");
	     exit(-1);
	 }
	 sds_id = SDselect(sd_id, sds_index);
	 status = SDgetinfo(sds_id, sds_name, &rank, dim_sizes, &num_type, &attributes);
	 if (status != 0) {        
            printf ("SDgetinfo failed.\n");
            HEprint (stdout, 0);
            exit (-1);
	 }
	 printf("number of scans %d\n",dim_sizes[0]);
         printf("rank %d num_type %d\n",rank, num_type);
	 edges[0] = 1;
	 edges[1] = 1;
	 edges[2] = 0;
	 status = SDreaddata(sds_id, start, NULL, edges, (VOIDP)(&altitude));
	 if (status != 0) {    
             printf("read altitude failed\n");
             exit(-1);
         }     
 	 status = SDendaccess(sds_id);
         CalipsoMetadata.SatNumber = 0;
         CalipsoMetadata.SatelliteAltitude = altitude;
         printf("altitude %f\n",CalipsoMetadata.SatelliteAltitude);
         
	 /* get latitude */
	 sds_index = SDnametoindex(sd_id, "Latitude");
	 if (sds_index == FAIL) {
	     printf("cannot find Latitude in file\n");
	     exit(-1);
	 }
	 sds_id = SDselect(sd_id, sds_index);
	 status = SDgetinfo(sds_id, sds_name, &rank, dim_sizes, &num_type, &attributes);
	 if (status != 0) {        
            printf ("SDgetinfo failed.\n");
            HEprint (stdout, 0);
            exit (-1);
	 }
    	 LatBuf = (float32 *) calloc(dim_sizes[0], sizeof(float32));
	 edges[0] = dim_sizes[0];
	 edges[1] = 1;
	 edges[2] = 0;
	 status = SDreaddata(sds_id, start, NULL, edges, (VOIDP)LatBuf);
	 if (status != 0) {    
             printf("read profile_time failed\n");
             exit(-1);
         }    	 
 	 status = SDendaccess(sds_id);
         printf("Starting Lat %f\n", LatBuf[0]);
	 /* get longitude */
	 sds_index = SDnametoindex(sd_id, "Longitude");
	 if (sds_index == FAIL) {
	     printf("cannot find Longitude in file\n");
	     exit(-1);
	 }
	 sds_id = SDselect(sd_id, sds_index);
	 status = SDgetinfo(sds_id, sds_name, &rank, dim_sizes, &num_type, &attributes);
	 if (status != 0) {        
            printf ("SDgetinfo failed.\n");
            HEprint (stdout, 0);
            exit (-1);
	 }
    	 LonBuf = (float32 *) calloc(dim_sizes[0], sizeof(float32));
	 edges[0] = dim_sizes[0];
	 edges[1] = 1;
	 edges[2] = 0;
	 status = SDreaddata(sds_id, start, NULL, edges, (VOIDP)LonBuf);
	 if (status != 0) {    
             printf("read profile_time failed\n");
             exit(-1);
         }    	 
 	 status = SDendaccess(sds_id);
         printf("Starting Lon %f\n", LonBuf[0]);

       	 CalipsoMetadata.StartNavPoint = 0;
	 CalipsoMetadata.NavPointInterval = 1;
         
	GSFInitNewWriteOnlyFile(&GSFdata, GSF_GENERIC, "Calipso LIDAR data"
	, CalipsoMetadata.Year, CalipsoMetadata.JulianDay, CalipsoMetadata.SatNumber, CalipsoMetadata.NumPix
	, CalipsoMetadata.NumNavPts, CalipsoMetadata.StartNavPoint, CalipsoMetadata.NavPointInterval
	, CalipsoMetadata.SatelliteAltitude, CalipsoMetadata.SpatialResolution);

	GSFAddChannel(&GSFdata, 0, 0.4, 0
	, GSF_FLOAT, 32, "Total_Attenuated_Backscatter", "Calipso LIDAR Colorbar");
	GSFSetHistogramFlag(&GSFdata, GSF_ON);
    	if (CommandLineInfo.overwritePromptFlag) {
	    GSFSetOverwritePromptOn(&GSFdata);
	}
	else {
	    GSFSetOverwritePromptOff(&GSFdata);
	}

        /* add table for lidar heights */
        heightTable = GSFAddTable(&(GSFdata.GSFHeader.TableList), 0, CalipsoMetadata.NumPix, "Lidar Data Altitudes");
        if (!heightTable) {
            printf("GSFAddTestTables: Error in Table read...exiting\n");
            exit(-1);
        }
        for (ind1=0;ind1<CalipsoMetadata.NumPix;ind1++) {
            GSFLoadTableEntryByKey(&(GSFdata.GSFHeader.TableList), 0, ind1
            , lidarHeights[ind1]);
        }
	
	setupLUT(&GSFdata);
	
	if (GSFOpenWriteOnlyFile(&GSFdata, OutputFileName)) {
	    printf("Cannot create file %s\n", OutputFileName);
	    exit(-1);	
	}

	/* allocate buffer for cloud mask */
	CalipsoData = calloc(CalipsoMetadata.NumPix,sizeof (float32));
	if (!CalipsoData) {
	    printf("cannot allocate CalipsoData\n");
	    exit (-1);
	}
	outputLine = (float *)calloc(CalipsoMetadata.NumPix,sizeof (float));
	if (!outputLine) {
	    printf("cannot allocate outputLine\n");
	    exit (-1);
	}
	
        linecnt=0;    
        /* read scans of data and break into single lines */
 	 /* info on lidar */
	sds_index = SDnametoindex(sd_id, "Total_Attenuated_Backscatter_532");
        if (sds_index == FAIL) {
	    printf("Total_Attenuated_Backscatter_532\n");
	    exit(-1);
	}
	sds_id = SDselect(sd_id, sds_index);

        for (ind1=0;ind1<nscans;ind1++) {

	    start[0] = ind1;
	    start[1] = 0;
	    start[2] = 0;
	    edges[0] = 1;
	    edges[1] = CalipsoMetadata.NumPix;
	    edges[2] = 1;

	    status = SDreaddata(sds_id, start, NULL, edges, (VOIDP)CalipsoData);
	    if (status != 0) {
		printf("SDreaddata: CalipsoData read error at line %d\n", ind1);
		exit(-1);
	    }

	    for (ind2=0;ind2<CalipsoMetadata.NumPix;ind2++) {
		outputLine[ind2] = CalipsoData[ind2];
                Lat[ind2] = LatBuf[ind1];
                Lon[ind2] = LonBuf[ind1];
	    }

	    /* load navigation info */
	    GSFLoadNavigation(&GSFdata, startTime[ind1], Lat, Lon);
	    GSFLoadChannel(&GSFdata, 0, outputLine);

	    /* write line of data to output file */
	    GSFWriteLine(&GSFdata);

	    linecnt++;
	    if ((linecnt%1000)==0) {
	       printf("line %d\n", linecnt);
	       GSFUpdateAllHistograms(&GSFdata);
	    }
        }

 	status = SDendaccess(sds_id);
	SDend(sd_id);
	Hclose(file_id);
	GSFFinishWritingFile(&GSFdata);
	printf("%d lines calibrated\n", linecnt);
	free(CalipsoData);
	free(outputLine);
	free(profile_time);
	free(Lat);
	free(Lon);
	free(LatBuf);
	free(LonBuf);
        free(startTime);
        free(lidarHeights);

}   
float32 **calloc2dFloat32Array(int dim1, int dim2) 
/*************************************************************************
**************************************************************************/
{
int ind1;
float32 **Buffer;

     Buffer = (float32 **)calloc(dim1, sizeof(float32 *));
     if (!Buffer) {
	 printf("calloc2dFloatArray cannot calloc for dim1\n");
	 exit(-1);
     }
     for (ind1=0;ind1<dim1;ind1++) {
	 Buffer[ind1] = (float32 *)calloc(dim2, sizeof(float32));
	 if (!Buffer[ind1]) {
	     printf("calloc2dFloatArray cannot calloc element %d for dim2\n", ind1);
	     exit(-1);
	 }
     }
     return Buffer;
    
}
void dealloc2dFloat32Array(float32 **Buffer, int dim1) 
/*************************************************************************
**************************************************************************/
{
int ind1;
     for (ind1=0;ind1<dim1;ind1++) {
	 free(Buffer[ind1]);
     }
     free(Buffer);
} 
void secToTime(double secondsInDay, int *hour, int *minute, double *second)
 /*************************************************************
  * 
  *************************************************************/
{
    int secondsPerHour = 3600;

    (*hour) = (int)secondsInDay/secondsPerHour;
    (*minute) = ((int)secondsInDay - (*hour) * secondsPerHour)/60;
    (*second) = secondsInDay - (double)((*hour) * secondsPerHour) - (double)((*minute) * 60);
}
double secToHrs(double secondsInDay)
 /*************************************************************
  * 
  *************************************************************/
{
    int secondsPerHour = 3600;

    return secondsInDay/secondsPerHour;
}
 int mmddyy_to_jday(int month, int day, int year)
  /*************************************************************
  * 
  *************************************************************/
{
     static int days[13] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 
                            31, 30, 31};
     static char *months[] = {"Jan","Feb","Mar","Apr","May","June","July","Aug","Sep","Oct","Nov","Dec"};
     int i, jday = 0, start = 1;
     int leap_year;
     
	
     leap_year = year%4 == 0 && year%100 != 0 || year%400 == 0;
     if (day > days[month]) {
	 if (month!=2) {
	     printf("Error:  invalid day %d in %s (%d)", day, months[month-1], days[month]);
	     exit(-1);
	 }
	 else {
	     if (day > days[month] + leap_year) {
		 printf("Error:  invalid day %d in %s (%d)", day, months[month-1], days[month] + leap_year);
		 exit(-1);
	     }
	     
	 }
	 
     }
     for(i=1; i<month; ++i)
     {
	 jday += days[i];
     }

     jday += day;
     if (month > 2) {
	 jday += leap_year;
     }
	
     return jday;
 }
void extractProfileTime(double profile_time, int *year, int *month, int *day, int *hour, int *minute, double *sec)
/*************************************************************************
**************************************************************************/
{

    int tempInt;
    double secondsInDay;        
    double fractionDay;

    *year = (int)profile_time/10000;
    *month = ((int)profile_time - (*year * 10000)) / 100;
    *day = (int)profile_time - (*year * 10000) - (*month * 100);
    
    tempInt = *year*10000 + *month*100 + *day;
    fractionDay = profile_time - (double) tempInt;
    secondsInDay = 86400.0 * fractionDay;
    
    secToTime(secondsInDay, hour, minute, sec);
    
}
void PrintCommandLineOptions(char *argv[])
/**********************************************************************************
 * 
 **********************************************************************************/
{
	printf("\n"); 
	printf("Usage: %s [-i <HDF CALIPSO input file>] [-o <GSF output file name>]\n", argv[0]);
	printf("          [-overwrite]\n");
	
	printf("\n");       
	printf("Required parameters:\n"); 
	printf("    -i <HDF level 1B input file> (must specify at least one)\n");
	printf("    -o <GSF output file name>\n");
	printf("\nOptional Parameters:\n");
	printf("    -overwrite do not prompt for overwrite of existing output file\n");

      (void) printf("\n"); 
    
}
void ProcessCommandLine(CommandLineStruct *poCommandLineInfo) {
/**********************************************************************************
 * 
 **********************************************************************************/
   int iArgIndex;
   int iState;
   enum {
    iNONE,
    iINPUT_FILE_NAME,
    iOUTPUT_FILE_NAME,
    iOVERWRITE
   };
    int numInputFiles =0;

/* set up defaults */
   poCommandLineInfo->OutputFileName = NULL;
   poCommandLineInfo->Errors = 0;
   poCommandLineInfo->overwritePromptFlag = 1;
   
   iState = iNONE;
   for (iArgIndex = 1; iArgIndex < poCommandLineInfo->Argc; iArgIndex++) {
      if (strcmp(poCommandLineInfo->Argv[iArgIndex], "-i") == 0) {
	 iState = iINPUT_FILE_NAME;
	 continue;
      }
      if (strcmp(poCommandLineInfo->Argv[iArgIndex], "-o") == 0) {
	 iState = iOUTPUT_FILE_NAME;
	 continue;
      }
      if (strcmp(poCommandLineInfo->Argv[iArgIndex], "-overwrite") == 0) {
	 iState = iOVERWRITE;
      }
      switch (iState) {
         default:
         case iNONE:
            poCommandLineInfo->Errors = 1;
	    break;
	 case iINPUT_FILE_NAME:
            poCommandLineInfo->InputFileName = strdup(poCommandLineInfo->Argv[iArgIndex]);
	    numInputFiles++;
	    iState = iNONE;
            break;
	 case iOUTPUT_FILE_NAME:
            poCommandLineInfo->OutputFileName = strdup(poCommandLineInfo->Argv[iArgIndex]);
	    iState = iNONE;
            break;
	 case iOVERWRITE:
	    poCommandLineInfo->overwritePromptFlag = 0;
	    break;
       }
    }
    /* check required parameters */
   if (numInputFiles != 1 ) poCommandLineInfo->Errors = 1;
   if (poCommandLineInfo->OutputFileName == NULL ) poCommandLineInfo->Errors = 1;
   return;
}
void setupLUT(GSFType *GSFdata)
/*********************************************************
 * 
 *********************************************************/
{
GSFDisplayLUTType *tempLUT;
GSFDisplayLUTType *LUT;
int lutIndex=0;

    tempLUT = GSFdata->GSFHeader.DisplayLUTHead;
    if (tempLUT) {
	while (tempLUT->NextDisplayLUT!=NULL) {
	    tempLUT = tempLUT->NextDisplayLUT;
	}
	lutIndex=tempLUT->DisplayLUTKeyValue + 1;
    }
    LUT = GSFAddLUT(GSFdata, lutIndex, 5, "MOD35");
    GSFLoadLUTEntry(LUT, 0, 255, 255, 255,"Cloud");
    GSFLoadLUTEntry(LUT, 1, 180, 180, 180,"Probably Cloud");
    GSFLoadLUTEntry(LUT, 2, 106, 176, 114,"Probably Clear");
    GSFLoadLUTEntry(LUT, 3, 0, 76, 9,"Clear");
    GSFLoadLUTEntry(LUT, 4, 255, 30, 0,"No Result");
}
